def safe_import_daily_performance(): ...
def safe_import_weight_backtest(): ...
def safe_import_top_drawdowns(): ...
def apply_stats_style(stats_df): ...
def ensure_datetime_index(df, dt_col: str = 'dt'): ...
def generate_component_key(data, prefix: str = 'component', **kwargs): ...



import pandas as pd
from _typeshed import Incomplete
from typing import Any

class HtmlReportBuilder:
    title: Incomplete
    theme: Incomplete
    sections: Incomplete
    custom_css: Incomplete
    custom_scripts: Incomplete
    chart_count: int
    def __init__(self, title: str = 'HTML 报告', theme: str = 'light') -> None: ...
    def add_custom_css(self, css: str) -> HtmlReportBuilder: ...
    def add_custom_script(self, script: str) -> HtmlReportBuilder: ...
    def add_header(self, params: dict[str, str], subtitle: str = None) -> HtmlReportBuilder: ...
    def add_metrics(self, metrics: list[dict[str, Any]], title: str = '核心绩效指标') -> HtmlReportBuilder: ...
    def add_chart_tab(self, name: str, chart_html: str, icon: str = 'bi-graph-up', active: bool = False) -> HtmlReportBuilder: ...
    def add_charts_section(self, title: str = '可视化分析') -> HtmlReportBuilder: ...
    def add_table(self, df: pd.DataFrame, title: str = '数据表', max_rows: int = None, style: str = 'Table Grid') -> HtmlReportBuilder: ...
    def add_section(self, title: str, content: str, icon: str = 'bi-file-text') -> HtmlReportBuilder: ...
    def add_footer(self, text: str = None) -> HtmlReportBuilder: ...
    def render(self) -> str: ...
    def save(self, file_path: str) -> str: ...



from .backtest import show_backtest_by_symbol as show_backtest_by_symbol, show_backtest_by_thresholds as show_backtest_by_thresholds, show_backtest_by_year as show_backtest_by_year, show_comprehensive_weight_backtest as show_comprehensive_weight_backtest, show_holds_backtest as show_holds_backtest, show_long_short_backtest as show_long_short_backtest, show_stoploss_by_direction as show_stoploss_by_direction, show_weight_backtest as show_weight_backtest, show_weight_distribution as show_weight_distribution
from .correlation import show_cointegration as show_cointegration, show_corr_graph as show_corr_graph, show_correlation as show_correlation, show_sectional_ic as show_sectional_ic, show_symbols_corr as show_symbols_corr, show_ts_rolling_corr as show_ts_rolling_corr, show_ts_self_corr as show_ts_self_corr
from .factor import show_event_features as show_event_features, show_event_return as show_event_return, show_factor_layering as show_factor_layering, show_factor_value as show_factor_value, show_feature_returns as show_feature_returns
from .forms import code_editor_form as code_editor_form, weight_backtest_form as weight_backtest_form
from .price_analysis import show_price_sensitive as show_price_sensitive
from .returns import show_cumulative_returns as show_cumulative_returns, show_daily_return as show_daily_return, show_drawdowns as show_drawdowns, show_monthly_return as show_monthly_return, show_rolling_daily_performance as show_rolling_daily_performance
from .statistics import show_classify as show_classify, show_date_effect as show_date_effect, show_describe as show_describe, show_df_describe as show_df_describe, show_normality_check as show_normality_check, show_out_in_compare as show_out_in_compare, show_outsample_by_dailys as show_outsample_by_dailys, show_psi as show_psi, show_splited_daily as show_splited_daily, show_yearly_stats as show_yearly_stats
from .strategy import show_cta_periods_classify as show_cta_periods_classify, show_czsc_trader as show_czsc_trader, show_multi_backtest as show_multi_backtest, show_optuna_study as show_optuna_study, show_portfolio as show_portfolio, show_quarterly_effect as show_quarterly_effect, show_returns_contribution as show_returns_contribution, show_stats_compare as show_stats_compare, show_strategies_recent as show_strategies_recent, show_symbol_penalty as show_symbol_penalty, show_symbols_bench as show_symbols_bench, show_turnover_rate as show_turnover_rate, show_volatility_classify as show_volatility_classify
from .utils import streamlit_run as streamlit_run
from .weights import show_weight_abs as show_weight_abs, show_weight_cdf as show_weight_cdf, show_weight_dist as show_weight_dist, show_weight_ts as show_weight_ts

__all__ = ['show_daily_return', 'show_cumulative_returns', 'show_monthly_return', 'show_drawdowns', 'show_rolling_daily_performance', 'show_correlation', 'show_sectional_ic', 'show_ts_rolling_corr', 'show_ts_self_corr', 'show_cointegration', 'show_corr_graph', 'show_symbols_corr', 'show_feature_returns', 'show_factor_layering', 'show_factor_value', 'show_event_return', 'show_event_features', 'show_weight_distribution', 'show_weight_backtest', 'show_holds_backtest', 'show_stoploss_by_direction', 'show_backtest_by_thresholds', 'show_backtest_by_year', 'show_backtest_by_symbol', 'show_long_short_backtest', 'show_comprehensive_weight_backtest', 'run_weight_backtest_app', 'show_splited_daily', 'show_yearly_stats', 'show_out_in_compare', 'show_outsample_by_dailys', 'show_psi', 'show_classify', 'show_date_effect', 'show_normality_check', 'show_describe', 'show_df_describe', 'show_optuna_study', 'show_czsc_trader', 'show_strategies_recent', 'show_returns_contribution', 'show_symbols_bench', 'show_quarterly_effect', 'show_cta_periods_classify', 'show_volatility_classify', 'show_portfolio', 'show_turnover_rate', 'show_stats_compare', 'show_symbol_penalty', 'show_multi_backtest', 'show_weight_ts', 'show_weight_dist', 'show_weight_cdf', 'show_weight_abs', 'streamlit_run', 'show_price_sensitive', 'weight_backtest_form', 'code_editor_form']

# Names in __all__ with no definition:
#   run_weight_backtest_app



import pandas as pd
from czsc.fsa.base import FeishuApiBase as FeishuApiBase, request as request
from czsc.fsa.bi_table import BiTable as BiTable
from czsc.fsa.im import IM as IM
from czsc.fsa.push_strategy_latest import push_strategy_latest as push_strategy_latest
from czsc.fsa.spreed_sheets import SingleSheet as SingleSheet, SpreadSheets as SpreadSheets

__all__ = ['request', 'FeishuApiBase', 'SpreadSheets', 'SingleSheet', 'read_feishu_sheet', 'update_spreadsheet', 'IM', 'push_text', 'push_card', 'push_message', 'get_feishu_members_by_mobiles', 'BiTable', 'push_strategy_latest']

def push_text(text: str, key: str) -> None: ...
def push_card(card: dict, key: str) -> None: ...
def read_feishu_sheet(spread_sheet_token: str, sheet_id: str = None, **kwargs): ...
def get_feishu_members_by_mobiles(mobiles: list, **kwargs): ...
def push_message(msg: str, msg_type: str = 'text', receive_id_type: str = 'open_id', **kwargs) -> None: ...
def update_spreadsheet(df: pd.DataFrame, spreadsheet_token: str, sheet_id: str, **kwargs) -> int: ...



from collections import OrderedDict
from czsc.core import BI as BI, CZSC as CZSC, Direction as Direction, FX as FX, Mark as Mark, RawBar as RawBar, Signal as Signal, ZS as ZS
from czsc.traders.base import CzscSignals as CzscSignals
from czsc.utils import single_linear as single_linear
from czsc.utils.sig import cal_cross_num as cal_cross_num, count_last_same as count_last_same, create_single_signal as create_single_signal, cross_zero_axis as cross_zero_axis, down_cross_count as down_cross_count, fast_slow_cross as fast_slow_cross, get_sub_elements as get_sub_elements
from deprecated import deprecated as deprecated

def update_ma_cache(c: CZSC, **kwargs): ...
def update_macd_cache(c: CZSC, **kwargs): ...
def update_boll_cache_V230228(c: CZSC, **kwargs): ...
def update_boll_cache(c: CZSC, **kwargs): ...
def tas_boll_vt_V230212(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_base_V221028(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_direct_V221106(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_power_V221108(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_first_bs_V221201(c: CZSC, **kwargs): ...
def tas_macd_first_bs_V221216(c: CZSC, **kwargs): ...
def tas_macd_second_bs_V221201(c: CZSC, **kwargs): ...
def tas_macd_xt_V221208(c: CZSC, **kwargs): ...
def tas_macd_bc_V221201(c: CZSC, **kwargs): ...
def tas_macd_change_V221105(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_ma_base_V221101(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_ma_base_V221203(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_ma_base_V230313(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_ma_round_V221206(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_double_ma_V221203(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_double_ma_V230511(c: CZSC, **kwargs): ...
def tas_boll_power_V221112(c: CZSC, **kwargs): ...
def tas_boll_bc_V221118(c: CZSC, **kwargs): ...
def update_kdj_cache(c: CZSC, **kwargs): ...
def tas_kdj_base_V221101(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_kdj_evc_V221201(c: CZSC, **kwargs) -> OrderedDict: ...
def update_rsi_cache(c: CZSC, **kwargs): ...
def tas_rsi_base_V230227(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_first_bs_V230217(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_second_bs_V230228(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_second_bs_V230303(c: CZSC, **kwargs): ...
def tas_hlma_V230301(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_boll_cc_V230312(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_bs1_V230312(c: CZSC, **kwargs): ...
def tas_macd_bs1_V230313(c: CZSC, **kwargs): ...
def tas_macd_base_V230320(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_dif_layer_V241010(c: CZSC, **kwargs) -> OrderedDict: ...
def update_cci_cache(c: CZSC, **kwargs): ...
def tas_cci_base_V230402(c: CZSC, **kwargs) -> OrderedDict: ...
def cci_decision_V240620(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_kdj_evc_V230401(c: CZSC, **kwargs) -> OrderedDict: ...
def update_atr_cache(c: CZSC, **kwargs): ...
def tas_atr_break_V230424(c: CZSC, **kwargs): ...
def update_sar_cache(c: CZSC, **kwargs): ...
def tas_sar_base_V230425(c: CZSC, **kwargs): ...
def tas_ma_system_V230513(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_bs1_V230411(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_bs1_V230412(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_accelerate_V230531(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_ma_cohere_V230512(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_cross_status_V230619(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_cross_status_V230624(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_cross_status_V230625(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_low_trend_V230627(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_atr_V230630(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_rumi_V230704(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_dist_V230408(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_dist_V230409(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_dist_V230410(c: CZSC, **kwargs) -> OrderedDict: ...
def cat_macd_V230518(cat: CzscSignals, **kwargs) -> OrderedDict: ...
def cat_macd_V230520(cat: CzscSignals, **kwargs) -> OrderedDict: ...
def tas_angle_V230802(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_bc_V230803(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_bc_V230804(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_bc_ubi_V230804(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_slope_V231019(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_double_ma_V240208(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_macd_bc_V240307(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_dma_bs_V240608(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_dif_zero_V240612(c: CZSC, **kwargs) -> OrderedDict: ...
def tas_dif_zero_V240614(c: CZSC, **kwargs) -> OrderedDict: ...



from .cache import DiskCache as DiskCache, clear_cache as clear_cache, clear_expired_cache as clear_expired_cache, disk_cache as disk_cache, empty_cache_path as empty_cache_path, get_dir_size as get_dir_size, home_path as home_path
from .client import DataClient as DataClient, get_url_token as get_url_token, set_url_token as set_url_token
from .converters import convert_dict_to_dataframe as convert_dict_to_dataframe, ensure_datetime_column as ensure_datetime_column, flatten_multiindex_columns as flatten_multiindex_columns, normalize_symbol as normalize_symbol, pivot_weight_data as pivot_weight_data, resample_to_period as resample_to_period, to_standard_kline_format as to_standard_kline_format
from .validators import validate_dataframe_columns as validate_dataframe_columns, validate_date_range as validate_date_range, validate_datetime_index as validate_datetime_index, validate_no_duplicates as validate_no_duplicates, validate_numeric_column as validate_numeric_column, validate_weight_data as validate_weight_data

__all__ = ['home_path', 'get_dir_size', 'empty_cache_path', 'DiskCache', 'disk_cache', 'clear_cache', 'clear_expired_cache', 'DataClient', 'set_url_token', 'get_url_token', 'validate_dataframe_columns', 'validate_datetime_index', 'validate_numeric_column', 'validate_date_range', 'validate_no_duplicates', 'validate_weight_data', 'to_standard_kline_format', 'pivot_weight_data', 'resample_to_period', 'normalize_symbol', 'convert_dict_to_dataframe', 'ensure_datetime_column', 'flatten_multiindex_columns']



"""
分析工具模块

包括统计分析、相关性分析和事件分析
"""

# 从 stats 导入统计函数
from .stats import (
    daily_performance,
    holds_performance,
    top_drawdowns,
    rolling_daily_performance,
    psi,
)

# 从 corr 导入相关性分析函数
from .corr import (
    nmi_matrix,
    single_linear,
    cross_sectional_ic,
)

# 从 events 导入事件分析函数
from .events import (
    overlap,
)

__all__ = [
    # Stats
    'daily_performance',
    'holds_performance',
    'top_drawdowns',
    'rolling_daily_performance',
    'psi',
    # Correlation
    'nmi_matrix',
    'single_linear',
    'cross_sectional_ic',
    # Events
    'overlap',
]



"""
Streamlit Visualize Components (SVC) - 缠中说禅可视化组件库

该模块提供了一套完整的 Streamlit 可视化组件，用于金融数据分析、策略回测、因子分析等场景。

主要功能模块：
- returns: 收益相关的可视化组件
- correlation: 相关性分析组件
- factor: 因子分析组件
- backtest: 回测相关组件
- statistics: 统计分析组件
- strategy: 策略分析组件
- weights: 持仓权重分析组件
- utils: 工具类组件
- forms: 用户输入表单组件
"""

from .returns import (
    show_daily_return,
    show_cumulative_returns,
    show_monthly_return,
    show_drawdowns,
    show_rolling_daily_performance,
)

from .correlation import (
    show_correlation,
    show_sectional_ic,
    show_ts_rolling_corr,
    show_ts_self_corr,
    show_cointegration,
    show_corr_graph,
    show_symbols_corr,
)

from .factor import (
    show_feature_returns,
    show_factor_layering,
    show_factor_value,
    show_event_return,
    show_event_features,
)


# 回测相关组件
from .backtest import (
    show_weight_distribution,
    show_weight_backtest,
    show_holds_backtest,
    show_stoploss_by_direction,
    show_backtest_by_thresholds,
    show_backtest_by_year,
    show_backtest_by_symbol,
    show_long_short_backtest,
    show_comprehensive_weight_backtest,
)

from .statistics import (
    show_splited_daily,
    show_yearly_stats,
    show_out_in_compare,
    show_outsample_by_dailys,
    show_psi,
    show_classify,
    show_date_effect,
    show_normality_check,
    show_describe,
    show_df_describe,
)

from .utils import (
    streamlit_run,
)

from .price_analysis import (
    show_price_sensitive,
)

from .strategy import (
    show_optuna_study,
    show_czsc_trader,
    show_strategies_recent,
    show_returns_contribution,
    show_symbols_bench,
    show_quarterly_effect,
    show_cta_periods_classify,
    show_volatility_classify,
    show_portfolio,
    show_turnover_rate,
    show_stats_compare,
    show_symbol_penalty,
    show_multi_backtest,
)

from .weights import (
    show_weight_ts,
    show_weight_dist,
    show_weight_cdf,
    show_weight_abs,
)

# 从 forms.py 导入所有表单函数
from .forms import (
    weight_backtest_form,
    code_editor_form
)


# 将所有函数添加到 __all__ 中
__all__ = [
    # 收益相关
    "show_daily_return",
    "show_cumulative_returns",
    "show_monthly_return",
    "show_drawdowns",
    "show_rolling_daily_performance",
    # 相关性分析
    "show_correlation",
    "show_sectional_ic",
    "show_ts_rolling_corr",
    "show_ts_self_corr",
    "show_cointegration",
    "show_corr_graph",
    "show_symbols_corr",
    # 因子分析
    "show_feature_returns",
    "show_factor_layering",
    "show_factor_value",
    "show_event_return",
    "show_event_features",
    # 回测相关
    "show_weight_distribution",
    "show_weight_backtest",
    "show_holds_backtest",
    "show_stoploss_by_direction",
    "show_backtest_by_thresholds",
    "show_backtest_by_year",
    "show_backtest_by_symbol",
    "show_long_short_backtest",
    "show_comprehensive_weight_backtest",
    "run_weight_backtest_app",
    # 统计分析
    "show_splited_daily",
    "show_yearly_stats",
    "show_out_in_compare",
    "show_outsample_by_dailys",
    "show_psi",
    "show_classify",
    "show_date_effect",
    "show_normality_check",
    "show_describe",
    "show_df_describe",
    # 策略分析
    "show_optuna_study",
    "show_czsc_trader",
    "show_strategies_recent",
    "show_returns_contribution",
    "show_symbols_bench",
    "show_quarterly_effect",
    "show_cta_periods_classify",
    "show_volatility_classify",
    "show_portfolio",
    "show_turnover_rate",
    "show_stats_compare",
    "show_symbol_penalty",
    "show_multi_backtest",
    # 权重分析
    "show_weight_ts",
    "show_weight_dist",
    "show_weight_cdf",
    "show_weight_abs",
    # 工具类
    "streamlit_run",
    # 价格敏感性分析
    "show_price_sensitive",
    # 表单组件
    "weight_backtest_form",
    "code_editor_form",
]



# -*- coding: utf-8 -*-
"""
author: zengbin93
email: zeng_bin8888@163.com
create_dt: 2023/06/16 19:45
describe: 飞书多维表格接口
"""
import os
import pandas as pd
from czsc.fsa.base import FeishuApiBase, request


class BiTable(FeishuApiBase):
    """
    多维表格概述: https://open.feishu.cn/document/server-docs/docs/bitable-v1/bitable-overview
    """

    def __init__(self, app_id=None, app_secret=None, app_token=None, **kwargs):
        """

        :param app_id: 飞书应用的唯一标识
        :param app_secret: 飞书应用的密钥
        :param app_token: 一个多维表格的唯一标识。示例值："bascnKMKGS5oD3lmCHq9euO8cGh"
        """
        app_id = app_id or os.getenv("FEISHU_APP_ID")
        app_secret = app_secret or os.getenv("FEISHU_APP_SECRET")
        super().__init__(app_id, app_secret, **kwargs)
        self.app_token = app_token
        # self.logger = kwargs.get("logger", loguru.logger)

    def one_record(self, table_id, record_id):
        """根据 record_id 的值检索现有记录

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-record/get
        """
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/{table_id}/records/{record_id}"
        return request("GET", url, self.get_headers())

    def list_records(self, table_id, **kwargs):
        """列出数据表中的记录

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-record/list

        :param table_id: 数据表id
        :return: 返回数据
        """
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/{table_id}/records"
        if kwargs.get("page_size") is None:
            kwargs["page_size"] = 500
        if kwargs.get("page_token") is None:
            kwargs["page_token"] = ""
        url = url + "?" + "&".join([f"{k}={v}" for k, v in kwargs.items()])
        return request("GET", url, self.get_headers())

    # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # 数据表相关api
    # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    def create_table(self, name=None, default_view_name=None, fields=None):
        """新增一个仅包含索引列的空数据表，也可以指定一部分初始字段。

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table/create
        :param name:非必填 数据表名称 请注意：
            名称中的首尾空格将会被去除。
            示例值："table1"
            数据校验规则：
            长度范围：1 字符 ～ 100 字符
        :param default_view_name:非必填 默认表格视图的名称，不填则默认为 表格。
        :param fields: 非必填 数据表的初始字段。数组类型
            结构：
            field_name： 必填 字段名
            type：必填	字段类型
            ui_type：字段在界面上的展示类型
            property：字段属性
            description： 字段的描述
        :return: 返回数据
        """
        params = {}
        if name is not None:
            params["name"] = name
        if default_view_name is not None:
            params["default_view_name"] = default_view_name
        if default_view_name is not None:
            params["fields"] = fields
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables"
        return request("POST", url, self.get_headers(), payload={"table": params})

    def batch_create_table(self, names=None):
        """新增多个数据表。

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table/batch_create
        :param names:非必填 数据表名称 []
            name :数据表名称
                    请注意：
                    名称中的首尾空格将会被去除。
                    示例值："table1"
                    数据校验规则：
                    长度范围：1 字符 ～ 100 字符
        :return: 返回数据
        """
        params = []
        if names is not None:
            for name in names:
                params.append({"name": name})
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/batch_create"
        return request("POST", url, self.get_headers(), payload={"tables": params})

    def delete_table(self, table_id):
        """删除一个数据表，最后一张数据表不允许被删除。

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table/delete
        :param table_id:多维表格数据表的唯一标识符

        :return: 返回数据
        """
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/{table_id}"
        return request("DELETE", url, self.get_headers())

    def batch_delete_table(self, table_ids=None):
        """删除一个数据表，最后一张数据表不允许被删除。

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table/batch_delete
        :param table_ids: 待删除的数据表的id [table_id 参数说明]，当前一次操作最多支持50个数据表

        :return: 返回数据
        """
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/batch_delete"
        return request("POST", url, self.get_headers(), payload={"table_ids": table_ids})

    def patch_table(self, table_id, name):
        """更新数据表的基本信息，包括数据表的名称等

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table/patch
        :param table_id: 多维表格数据表的唯一标识符
        :param name: 数据表的新名称。请注意：
                    名称中的首尾空格将会被去除。
                    如果名称为空或和旧名称相同，接口仍然会返回成功，但是名称不会被更改。
                    示例值："数据表的新名称"
                    数据校验规则：
                    长度范围：1 字符 ～ 100 字符
        :return: 返回数据
        """
        params = {}
        if name is not None:
            params["name"] = name
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/{table_id}"
        return request("PATCH", url, self.get_headers(), payload=params)

    def list_tables(self, page_token=None, page_size=20):
        """获取多维表格下的所有数据表。

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table/list
        :param page_token: 	分页标记，第一次请求不填，表示从头开始遍历；分页查询结果还有更多项时会同时返回新的 page_token，
            下次遍历可采用该 page_token 获取查询结果
        :param page_size: 	分页大小示例值：10 默认值：20 数据校验规则：最大值：100
        :return: 返回数据
        """
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables?page_size={page_size}"
        url = url if page_token is None else url + f"&page_token={page_token}"
        return request("GET", url, self.get_headers())

    # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # 记录相关api
    # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    def table_record_get(
        self,
        table_id,
        record_id,
        text_field_as_array=None,
        user_id_type=None,
        display_formula_ref=None,
        with_shared_url=None,
        automatic_fields=None,
    ):
        """获取多维表格下的所有数据表。

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-record/get
        :param table_id: table id
        :param record_id: 单条记录的 id
        :param text_field_as_array: 非必需 多行文本字段数据是否以数组形式返回。true 表示以数组形式返回。默认为 false
        :param user_id_type: 非必需 用户 ID 类型
        :param display_formula_ref: 控制公式、查找引用是否显示完整原样的返回结果。默认为 false
        :param with_shared_url: 非必需 	控制是否返回该记录的链接，即 record_url 参数。默认为 false，即不返回
        :param automatic_fields: 非必需 控制是否返回自动计算的字段，例如 created_by/created_time/last_modified_by/last_modified_time，true 表示返回。默认为 false

        :return: 返回数据
        """
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/{table_id}/records/{record_id}?1=1"
        url = url if text_field_as_array is None else url + f"&text_field_as_array={text_field_as_array}"
        url = url if user_id_type is None else url + f"&user_id_type={user_id_type}"
        url = url if display_formula_ref is None else url + f"&display_formula_ref={display_formula_ref}"
        url = url if with_shared_url is None else url + f"&with_shared_url={with_shared_url}"
        url = url if automatic_fields is None else url + f"&automatic_fields={automatic_fields}"
        return request("GET", url, self.get_headers())

    def table_record_search(
        self,
        table_id,
        user_id_type=None,
        page_token=None,
        page_size=20,
        view_id=None,
        field_names=None,
        sort=None,
        filter=None,
        automatic_fields=None,
    ):
        """查询数据表中的现有记录，单次最多查询 500 行记录，支持分页获取

        https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/reference/bitable-v1/app-table-record/search
        :param table_id: table id
        :param user_id_type: 非必需	用户 ID 类型
        :param page_token: 非必需 分页标记，第一次请求不填，表示从头开始遍历；分页查询结果还有更多项时会同时返回新的 page_token，下次遍历可采用该 page_token 获取查询结果
        :param page_size: 非必需 分页大小。最大值为 500

        :param view_id: 非必需 视图的唯一标识符，获取指定视图下的记录view_id 参数说明 注意：当 filter 参数 或 sort 参数不为空时，请求视为对数据表中的全部数据做条件过滤，指定的view_id 会被忽略。数据校验规则：长度范围：0 字符 ～ 50 字符
        :param field_names: 非必需 字段名称，用于指定本次查询返回记录中包含的字段
        :param sort: 非必需 sort[] 排序条件
                        field_name: 非必需 字段名称 示例值："多行文本"  数据校验规则：长度范围：0 字符 ～ 1000 字符
                        desc:非必需 是否倒序排序 示例值：true 默认值：false
        :param filter: 非必需 筛选条件
        :param automatic_fields: 非必需 控制是否返回自动计算的字段, true 表示返回

        :return: 返回数据
        """
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/{table_id}/records/search?page_size={page_size}"
        url = url if user_id_type is None else url + f"&user_id_type={user_id_type}"
        url = url if page_token is None else url + f"&page_token={page_token}"

        params = {}
        if view_id is not None:
            params["view_id"] = view_id
        if field_names is not None:
            params["field_names"] = field_names
        if sort is not None:
            params["sort"] = sort
        if filter is not None:
            params["filter"] = filter
        if automatic_fields is not None:
            params["automatic_fields"] = automatic_fields
        return request("POST", url, self.get_headers(), payload=params)

    def table_record_create(self, table_id, fields, user_id_type=None, client_token=None):
        """数据表中新增一条记录

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-record/create

        :param table_id: table id
        :param user_id_type: 非必需 用户 ID 类型
        :param client_token: 非必需 格式为标准的 uuidv4，操作的唯一标识，用于幂等的进行更新操作。此值为空表示将发起一次新的请求，此值非空表示幂等的进行更新操作。
        :param fields: 必需
            数据表的字段，即数据表的列。当前接口支持的字段类型为：多行文本、单选、条码、多选、日期、人员、附件、复选框、超链接、数字、单向关联、双向关联、电话号码、地理位置。详情参考
        :return: 返回数据
        """
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/{table_id}/records?1=1"
        url = url if user_id_type is None else url + f"&user_id_type={user_id_type}"
        url = url if client_token is None else url + f"&client_token={client_token}"
        return request("POST", url, self.get_headers(), payload={"fields": fields})

    def table_record_update(self, table_id, record_id, fields, user_id_type=None):
        """更新数据表中的一条记录

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-record/update
        :param table_id: table id
        :param record_id: 一条记录的唯一标识 id

        :param user_id_type: 非必需 用户 ID 类型

        :param fields: 必需
            数据表的字段，即数据表的列。当前接口支持的字段类型为：多行文本、单选、条码、多选、日期、人员、附件、复选框、超链接、数字、单向关联、双向关联、电话号码、地理位置。详情参考

        :return: 返回数据
        """
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/{table_id}/records/{record_id}/?1=1"
        url = url if user_id_type is None else url + f"&user_id_type={user_id_type}"
        return request("PUT", url, self.get_headers(), payload={"fields": fields})

    def table_record_delete(self, table_id, record_id):
        """删除数据表中的一条记录

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-record/delete
        :param table_id: table id
        :param record_id: 一条记录的唯一标识 id
        :return: 返回数据
        """
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/{table_id}/records/{record_id}"
        return request("DELETE", url, self.get_headers())

    def table_record_batch_create(self, table_id, records, user_id_type=None, client_token=None):
        """在数据表中新增多条记录，单次调用最多新增 500 条记录。

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-record/batch_create

        :param table_id: table id
        :param user_id_type: 非必需 用户 ID 类型
        :param client_token: 非必需 格式为标准的 uuidv4，操作的唯一标识，用于幂等的进行更新操作。此值为空表示将发起一次新的请求，此值非空表示幂等的进行更新操作。
        :param records:[] 	数据表的字段，即数据表的列当前接口支持的字段类型 示例值：{"多行文本":"HelloWorld"}
        :return: 返回数据
        """
        # records = []
        # for field in fields:
        #     records.append({"fields": field})
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/{table_id}/records/batch_create?1=1"
        url = url if user_id_type is None else url + f"&user_id_type={user_id_type}"
        url = url if client_token is None else url + f"&client_token={client_token}"
        return request("POST", url, self.get_headers(), payload={"records": records})

    def table_record_batch_update(self, table_id, records, user_id_type=None):
        """更新数据表中的多条记录，单次调用最多更新 500 条记录。

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-record/batch_update
        :param table_id: table id

        :param user_id_type: 非必需 用户 ID 类型

        :param records:[] 	记录
                        [{
                            "record_id": "reclAqylTN",
                            "fields": {
                                "索引": "索引列多行文本类型"
                            }
                        }]
        :return: 返回数据
        """
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/{table_id}/records/batch_update?1=1"
        url = url if user_id_type is None else url + f"&user_id_type={user_id_type}"
        return request("POST", url, self.get_headers(), payload={"records": records})

    def table_record_batch_delete(self, table_id, record_ids):
        """删除数据表中现有的多条记录，单次调用中最多删除 500 条记录

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-record/batch_delete
        :param table_id: table id

        :param record_ids:string[] 	删除的多条记录id列表示例值：["recwNXzPQv"]

        :return: 返回数据
        """
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/{table_id}/records/batch_delete"
        return request("POST", url, self.get_headers(), payload={"records": record_ids})

    # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # 视图相关api
    # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    def table_view_patch(self, table_id, view_id, infos):
        """增量修改视图信息

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-view/patch
        :param table_id: table id
        :param view_id: 视图 ID

        :param infos: 修改信息
                    view_name: 	视图名称
                    property: 非必需	视图属性
        :return: 返回数据
        """
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/{table_id}/views/{view_id}"
        return request("PATCH", url, self.get_headers(), payload=infos)

    def table_view_get(self, table_id, view_id):
        """根据 view_id 检索现有视图

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-view/get
        :param table_id: table id
        :param view_id: 视图 ID

        :return: 返回数据
        """
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/{table_id}/views/{view_id}"
        return request("GET", url, self.get_headers())

    def table_view_list(self, table_id, page_size=20, user_id_type=None, page_token=None):
        """根据 app_token 和 table_id，获取数据表的所有视图

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-view/list
        :param table_id: table id

        :param user_id_type: 非必需 用户 ID 类型 示例值："open_id"
        :param page_token:非必需 	分页标记，第一次请求不填，表示从头开始遍历；分页查询结果还有更多项时会同时返回新的 page_token，下次遍历可采用该 page_token 获取查询结果
        :param page_size:非必需 分页大小 示例值：10 默认值：20 数据校验规则：最大值：100

        :return: 返回数据
        """
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/{table_id}/views?page_size={page_size}"
        url = url if user_id_type is None else url + f"&user_id_type={user_id_type}"
        url = url if page_token is None else url + f"&page_token={page_token}"
        return request("GET", url, self.get_headers())

    def table_view_create(self, table_id, view_name, view_type):
        """在数据表中新增一个视图

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-view/create
        :param table_id: table id

        :param view_name: 视图名字
        :param view_type: 视图类型 示例值："grid"
                        可选值有：
                            grid:表格视图
                            kanban:看板视图
                            gallery:画册视图
                            gantt:甘特视图
                            form:表单视图
        :return: 返回数据
        """
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/{table_id}/views"
        return request("POST", url, self.get_headers(), payload={"view_name": view_name, "view_type": view_type})

    def table_view_delete(self, table_id, view_id):
        """在数据表中新增一个视图

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-view/create
        :param table_id: table id

        :param view_id: 视图id

        :return: 返回数据
        """
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/{table_id}/views/{view_id}"
        return request("DELETE", url, self.get_headers())

    # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # 字段相关api
    # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    def table_field_list(self, table_id, page_size=20, view_id=None, text_field_as_array=None, page_token=None):
        """根据 app_token 和 table_id，获取数据表的所有字段

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-field/list
        :param table_id: table id

        :param view_id: 视图 ID
        :param text_field_as_array: 控制字段描述（多行文本格式）数据的返回格式, true 表示以数组富文本形式返回
        :param page_token: 	分页标记，第一次请求不填，表示从头开始遍历；分页查询结果还有更多项时会同时返回新的 page_token，下次遍历可采用该 page_token 获取查询结果
        :param page_size: 分页大小 示例值：10 默认值：20 数据校验规则：最大值：100

        :return: 返回数据
        """
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/{table_id}/fields?page_size={page_size}"
        url = url if view_id is None else url + f"&view_id={view_id}"
        url = url if text_field_as_array is None else url + f"&text_field_as_array={text_field_as_array}"
        url = url if page_token is None else url + f"&page_token={page_token}"
        return request("GET", url, self.get_headers())

    def table_field_create(
        self, table_id, field_name, type, property=None, description=None, ui_type=None, client_token=None
    ):
        """在数据表中新增一个字段

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-field/create
        :param table_id: table id

        :param client_token: 格式为标准的 uuidv4，操作的唯一标识，用于幂等的进行更新操作。此值为空表示将发起一次新的请求，此值非空表示幂等的进行更新操作。

        :param field_name: 	多维表格字段名
        :param type: 	多维表格字段类型
                        可选值有：1：多行文本 2：数字 3：单选 4：多选5：日期7：复选框11：人员13：电话号码15：超链接17：附件18：关联20：公式21：双向关联22：地理位置23：群组1001：创建时间1002：最后更新时间1003：创建人1004：修改人1005：自动编号
        :param property: 字段属性
        :param description: 字段的描述
        :param ui_type: 字段在界面上的展示类型，例如进度字段是数字的一种展示形态
                        示例值："Progress"
                        可选值有：Text：多行文本 Email：邮箱地址 Barcode：条码 Number：数字 Progress：进度 Currency：货币 Rating：评分 SingleSelect：单选 MultiSelect：多选 DateTime：日期 Checkbox：复选框 User：人员 GroupChat：群组 Phone：电话号码 Url：超链接 Attachment：附件 SingleLink：单向关联 Formula：公式 DuplexLink：双向关联 Location：地理位置 CreatedTime：创建时间 ModifiedTime：最后更新时间 CreatedUser：创建人 ModifiedUser：修改人 AutoNumber：自动编号
        :return: 返回数据
        """
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/{table_id}/fields?1=1"
        url = url if client_token is None else url + f"&client_token={client_token}"

        params = {}
        if field_name is not None:
            params["field_name"] = field_name
        if type is not None:
            params["type"] = type
        if property is not None:
            params["property"] = property
        if description is not None:
            params["description"] = description
        if ui_type is not None:
            params["ui_type"] = ui_type

        return request("POST", url, self.get_headers(), payload=params)

    def table_field_update(self, table_id, field_id, field_name, type, property=None, description=None, ui_type=None):
        """数据表中更新一个字段

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-field/update
        :param table_id: table id

        :param field_id: field id

        :param field_name: 	多维表格字段名
        :param type: 	多维表格字段类型
                        可选值有：1：多行文本 2：数字 3：单选 4：多选5：日期7：复选框11：人员13：电话号码15：超链接17：附件18：关联20：公式21：双向关联22：地理位置23：群组1001：创建时间1002：最后更新时间1003：创建人1004：修改人1005：自动编号
        :param property: 字段属性
        :param description: 字段的描述
        :param ui_type: 字段在界面上的展示类型，例如进度字段是数字的一种展示形态
                        示例值："Progress"
                        可选值有：Text：多行文本 Email：邮箱地址 Barcode：条码 Number：数字 Progress：进度 Currency：货币 Rating：评分 SingleSelect：单选 MultiSelect：多选 DateTime：日期 Checkbox：复选框 User：人员 GroupChat：群组 Phone：电话号码 Url：超链接 Attachment：附件 SingleLink：单向关联 Formula：公式 DuplexLink：双向关联 Location：地理位置 CreatedTime：创建时间 ModifiedTime：最后更新时间 CreatedUser：创建人 ModifiedUser：修改人 AutoNumber：自动编号
        :return: 返回数据
        """
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/{table_id}/fields/{field_id}"

        params = {}
        if field_name is not None:
            params["field_name"] = field_name
        if type is not None:
            params["type"] = type
        if property is not None:
            params["property"] = property
        if description is not None:
            params["description"] = description
        if ui_type is not None:
            params["ui_type"] = ui_type

        return request("PUT", url, self.get_headers(), payload=params)

    def table_field_delete(self, table_id, field_id):
        """数据表中删除一个字段

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-field/delete
        :param table_id: table id

        :param field_id: field id

        :return: 返回数据
        """
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/{table_id}/fields/{field_id}"
        return request("DELETE", url, self.get_headers())

    # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # 表单相关api
    # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    def table_form_patch_2(
        self, table_id, form_id, name=None, description=None, shared=None, shared_limit=None, submit_limit_once=None
    ):
        """更新表单中的元数据项

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/form/patch-2
        :param table_id: table id

        :param field_id: field id

        :param name: 非必需 表单名称
        :param description: 非必需 表单描述
        :param shared: 非必需 是否开启共享
        :param shared_limit: 非必需 分享范围限制 示例值："tenant_editable" 可选值有：off：仅邀请的人可填写 tenant_editable：组织内获得链接的人可填写 anyone_editable：互联网上获得链接的人可填写
        :param submit_limit_once: 非必需 填写次数限制一次 示例值：true

        :return: 返回数据
        """
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/{table_id}/forms/{form_id}"
        params = {}
        if name is not None:
            params["name"] = name
        if description is not None:
            params["description"] = description
        if shared is not None:
            params["shared"] = shared
        if shared_limit is not None:
            params["shared_limit"] = shared_limit
        if submit_limit_once is not None:
            params["submit_limit_once"] = submit_limit_once
        return request("PATCH", url, self.get_headers(), payload=params)

    def table_form_get(self, table_id, form_id):
        """列出表单的所有问题项

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/form/list
        :param table_id: table id

        :param field_id: field id

        :return: 返回数据
        """
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/{table_id}/forms/{form_id}"
        return request("GET", url, self.get_headers())

    def table_form_patch(
        self, table_id, form_id, field_id, pre_field_id=None, title=None, description=None, required=None, visible=None
    ):
        """更新表单中的问题项

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/form/patch
        :param table_id: table id

        :param field_id: field id

        :param pre_field_id: 非必需 上一个表单问题 ID，用于支持调整表单问题的顺序，通过前一个表单问题的 field_id 来确定位置；如果 pre_field_id 为空字符串，则说明要排到首个表单问题
        :param title: 非必需 	表单问题
        :param description: 非必需 	问题描述
        :param required: 非必需 是否必填
        :param visible: 非必需 是否可见，当值为 false 时，不允许更新其他字段

        :return: 返回数据
        """
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/{table_id}/forms/{form_id}/fields/{field_id}"
        params = {}
        if pre_field_id is not None:
            params["pre_field_id"] = pre_field_id
        if description is not None:
            params["description"] = description
        if title is not None:
            params["title"] = title
        if required is not None:
            params["required"] = required
        if visible is not None:
            params["visible"] = visible
        return request("PATCH", url, self.get_headers(), payload=params)

    def table_form_list(self, table_id, form_id):
        """列出表单的所有问题项

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/form/list
        :param table_id: table id

        :param field_id: field id

        :return: 返回数据
        """
        url = f"{self.host}/open-apis/bitable/v1/apps/{self.app_token}/tables/{table_id}/forms/{form_id}/fields"
        return request("GET", url, self.get_headers())

    # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # 多维表格相关api
    # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    def table_copy(self, app_token=None, name=None, folder_token=None, without_content=None, time_zone=None):
        """复制一个多维表格，可以指定复制到某个有权限的文件夹下

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app/copy

        :param app_token: app_token 不传复制当前表格
        :param name: 多维表格 App 名字
        :param folder_token: 多维表格 App 归属文件夹
        :param without_content: 是否复制多维表格内容，取值：true：不复制 false：复制
        :param time_zone: 文档时区 示例值："Asia/Shanghai"
        :return: 返回数据
        """
        if app_token is None:
            app_token = self.app_token
        url = f"{self.host}/open-apis/bitable/v1/apps/{app_token}/copy"
        params = {}
        if name is not None:
            params["name"] = name
        if folder_token is not None:
            params["folder_token"] = folder_token
        if without_content is not None:
            params["without_content"] = without_content
        if time_zone is not None:
            params["time_zone"] = time_zone
        return request("POST", url, self.get_headers(), payload=params)

    def table_create(self, name=None, folder_token=None, time_zone=None):
        """复制一个多维表格，可以指定复制到某个有权限的文件夹下

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app/create
        :param name: 多维表格 App 名字
        :param folder_token: 多维表格 App 归属文件夹
        :param time_zone: 文档时区 示例值："Asia/Shanghai"

        :return: 返回数据
        """
        url = f"{self.host}/open-apis/bitable/v1/apps"
        params = {}
        if name is not None:
            params["name"] = name
        if folder_token is not None:
            params["folder_token"] = folder_token
        if time_zone is not None:
            params["time_zone"] = time_zone
        return request("POST", url, self.get_headers(), payload=params)

    def table_get(self, app_token=None):
        """复制一个多维表格，可以指定复制到某个有权限的文件夹下

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app/get

        :param app_token: 不传获取当前表格
        :return: 返回数据
        """
        if app_token is None:
            app_token = self.app_token
        url = f"{self.host}/open-apis/bitable/v1/apps/{app_token}"
        return request("GET", url, self.get_headers())

    def table_update(self, app_token=None, name=None, is_advanced=None):
        """通过 app_token 更新多维表格元数据

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app/update
        :param app_token: 不传修改当前表格
        :param name: 新的多维表格名字
        :param is_advanced: 多维表格是否开启高级权限

        :return: 返回数据
        """
        if app_token is None:
            app_token = self.app_token
        url = f"{self.host}/open-apis/bitable/v1/apps/{app_token}"

        params = {}
        if name is not None:
            params["name"] = name
        if is_advanced is not None:
            params["is_advanced"] = is_advanced
        return request("PUT", url, self.get_headers(), payload=params)

    # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # 以下是便捷使用的封装，非官方API接口
    # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    @property
    def tables(self):
        """获取所有表格

        :return:
        """
        res = self.list_tables()
        return res["data"]["items"]

    def read_table(self, table_id, **kwargs):
        """读取多维表格中指定表格的数据

        :param table_id: 表格id
        :return:
        """
        rows = []
        res = self.list_records(table_id, **kwargs)["data"]
        total = res["total"]

        if total == 0:
            return pd.DataFrame()

        rows.extend(res["items"])
        while res["has_more"]:
            res = self.list_records(table_id, page_token=res["page_token"], **kwargs)["data"]
            rows.extend(res["items"])

        assert len(rows) == total, "数据读取异常"
        return pd.DataFrame([x["fields"] for x in rows])

    def empty_table(self, table_id, **kwargs):
        """清空多维表格中指定表格的数据，保留表头

        :param table_id: 表格id
        :return:
        """
        res = self.list_records(table_id, **kwargs)["data"]
        self.logger.info(f"{table_id} 表格中共有 {res['total']} 条数据")
        records = res["items"]
        if records:
            record_ids = [x["record_id"] for x in records]
            self.table_record_batch_delete(table_id, record_ids)
            self.logger.info(f"{table_id} 删除 {len(record_ids)} 条数据")

        while res["has_more"]:
            res = self.list_records(table_id, page_token=res["page_token"], **kwargs)["data"]
            records = res["items"]
            if records:
                record_ids = [x["record_id"] for x in records]
                self.table_record_batch_delete(table_id, record_ids)
                self.logger.info(f"{table_id} 删除 {len(record_ids)} 条数据")



from _typeshed import Incomplete
from czsc.traders.base import generate_czsc_signals as generate_czsc_signals

class DummyBacktest:
    strategy: Incomplete
    results_path: Incomplete
    signals_path: Incomplete
    poss_path: Incomplete
    read_bars: Incomplete
    kwargs: Incomplete
    sdt: Incomplete
    edt: Incomplete
    bars_sdt: Incomplete
    def __init__(self, strategy, signals_path, results_path, read_bars, **kwargs) -> None: ...
    def replay(self, symbol) -> None: ...
    def one_symbol_dummy(self, symbol) -> None: ...
    def one_pos_stats(self, pos_name): ...
    def execute(self, symbols, n_jobs: int = 2, **kwargs) -> None: ...



import pandas as pd
from _typeshed import Incomplete

class WordWriter:
    file_docx: Incomplete
    document: Incomplete
    def __init__(self, file_docx=None) -> None: ...
    def add_title(self, text) -> None: ...
    def add_heading(self, text, level: int = 1) -> None: ...
    def add_paragraph(self, text, style=None, bold: bool = False, first_line_indent: float = 0.74) -> None: ...
    def add_df_table(self, df: pd.DataFrame, style: str = 'Table Grid', **kwargs): ...
    def add_picture(self, file, width=None, height=None, alignment: str = 'center') -> None: ...
    def add_page_break(self) -> None: ...
    def save(self, file_docx=None) -> None: ...

def test_word_writer() -> None: ...



from czsc.fsa.base import FeishuApiBase as FeishuApiBase, request as request

class IM(FeishuApiBase):
    def __init__(self, app_id, app_secret, **kwargs) -> None: ...
    def get_user_id(self, payload, user_id_type: str = 'open_id'): ...
    def upload_im_file(self, file_path, file_type: str = 'stream'): ...
    def upload_im_image(self, image_path, image_type: str = 'message'): ...
    def send(self, payload, receive_id_type: str = 'open_id'): ...
    def send_text(self, text, receive_id, receive_id_type: str = 'open_id'): ...
    def send_card(self, card, receive_id, receive_id_type: str = 'open_id'): ...
    def send_image(self, image_path, receive_id, receive_id_type: str = 'open_id'): ...
    def send_file(self, file_path, receive_id, receive_id_type: str = 'open_id'): ...



from collections import OrderedDict
from czsc import CZSC as CZSC
from czsc.core import Direction as Direction, RawBar as RawBar, Signal as Signal
from czsc.utils.sig import get_sub_elements as get_sub_elements

def jcc_san_xing_xian_V221023(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_ten_mo_V221028(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_wu_yun_gai_ding_V221101(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_ci_tou_V221101(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_san_fa_V20221118(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_san_fa_V20221115(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_xing_xian_V221118(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_fen_shou_xian_V20221113(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_zhu_huo_xian_V221027(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_yun_xian_V221118(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_ping_tou_V221113(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_zhuo_yao_dai_xian_v221113(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_two_crow_V221108(c: CZSC, **kwargs): ...
def jcc_three_crow_V221108(c: CZSC, **kwargs): ...
def check_szx(bar: RawBar, th: int = 10, **kwargs) -> bool: ...
def jcc_szx_V221111(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_san_szx_V221122(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_fan_ji_xian_V221121(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_shan_chun_V221121(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_gap_yin_yang_V221121(c: CZSC, **kwargs) -> OrderedDict: ...
def jcc_ta_xing_V221124(c: CZSC, **kwargs) -> OrderedDict: ...



from .ret import RET001 as RET001, RET002 as RET002, RET003 as RET003, RET004 as RET004, RET005 as RET005, RET006 as RET006, RET007 as RET007, RET008 as RET008
from .vpf import VPF001 as VPF001, VPF002 as VPF002, VPF003 as VPF003, VPF004 as VPF004



import pandas as pd
from _typeshed import Incomplete
from czsc import Freq as Freq, RawBar as RawBar

cache_path: Incomplete
dc: Incomplete

def format_kline(kline: pd.DataFrame, freq: Freq) -> list[RawBar]: ...
def get_sw_members(level: str = 'L1'): ...
def get_daily_basic(sdt: str = '20100101', edt: str = '20240101'): ...
def moneyflow_hsgt(start_date, end_date): ...
def pro_bar_minutes(ts_code, sdt, edt, freq: str = '60min', asset: str = 'E', adj=None): ...
def get_symbols(step: str = 'all'): ...
def get_raw_bars(symbol, freq, sdt, edt, fq: str = '后复权', raw_bar: bool = True): ...



import pandas as pd
from _typeshed import Incomplete

def set_url_token(token, url, **kwargs) -> None: ...
def get_url_token(url, **kwargs): ...

class DataClient:
    __version__: str
    logger: Incomplete
    cache_path: Incomplete
    verbose: Incomplete
    def __init__(self, token: str | None = None, url: str = 'http://api.tushare.pro', timeout: int = 300, verbose: bool = False, **kwargs) -> None: ...
    def clear_cache(self, **kwargs) -> None: ...
    def clear_api_cache(self, api_name: str, expiration: int = -1) -> None: ...
    def post_request(self, api_name: str, fields: str = '', **kwargs) -> pd.DataFrame: ...
    def __getattr__(self, name): ...



from _typeshed import Incomplete

valid_true: Incomplete

def use_python(): ...
def get_verbose(verbose=None): ...
def get_welcome(): ...
def get_min_bi_len(v: int = None) -> int: ...
def get_max_bi_num(v: int = None) -> int: ...
